bashmatic = bashmatic or nil

function handleInstall(_, name)
    if name:lower() == "bashmatic" then
        if bashmatic == nil then
            -- it's empty
            bashmatic = {}
            bashmatic.commands = {
                "start" = nil,
                "before" = nil,
                "during" = nil,
                "duringTime" = 2,
                "after" = nil
            }
            bashmatic.death = {
                "phrase" = nil,
                "event" = nil
            }
            bashmatic.configs = {
                "enemies" = {
                    "detection" = "inline",
                    "location" = nil,
                    "data" = nil
                },
                "areas" = {
                    "detection" = nil,
                    "location" = nil,
                    "data" = nil
                }
            }
            bashmatic.enabled = false
            bashmatic.bashing = false
            bashmatic.hunting = {
                "targets" = {},
                "events" = {}
            }
        else
            -- it's not empty, verify install
            local corruptions = 0
            if not table.contains(bashmatic, "commands") then
                bashmatic.commands = {
                    "start" = nil,
                    "before" = nil,
                    "during" = nil,
                    "duringTime" = 2,
                    "after" = nil
                }
                cecho("<firebrick>BASHMATIC: Potentially corrupt install. Tried to repair bashing commands.<reset>")
                corruptions += 1
            end

            if not table.contains(bashmatic, "death") then
                bashmatic.death = {
                    "phrase" = nil,
                    "event" = nil
                }
                cecho("<firebrick>BASHMATIC: Potentially corrupt install. Tried to repair death trigger.<reset>")
                corruptions += 1
            end

            if not table.contains(bashmatic, "configs") then
                bashmatic.configs = {
                    "enemies"{
                        "detection" = "inline",
                        "location" = nil,
                        "data" = nil
                    },
                    "areas" = {
                        "detection" = nil,
                        "location" = nil,
                        "data" = nil
                    }
                }
                cecho("<firebrick>BASHMATIC: Potentially corrupt install. Tried to repair configuration.<reset>")
                corruptions += 1
            end

            if not table.contains(bashmatic, "enabled") then
                bashmatic.enabled = false
                cecho("<firebrick>BASHMATIC: Potentially corrupt install. Tried to repair status settings.<reset>")
                corruptions += 1
            end

            if not table.contains(bashmatic, "bashing") then
                bashmatic.bashing = false
                cecho("<firebrick>BASHMATIC: Potentially corrupt install. Tried to repair status settings.<reset>")
                corruptions += 1
            end

            if not table.contains(bashmatic, "hunting") then
                bashmatic.hunting = {
                    "targets" = {},
                    "events" = {}
                }
                cecho("<firebrick>BASHMATIC: Potentially corrupt install. Tried to repair target listings.<reset>")
                corruptions += 1
            end

            if corruptions >= 2 then
                cecho("<firebrick>BASHMATIC: Multiple corruptions fixed. You should probably uninstall and reinstall Bashmatic.<reset>")
            end
        end
    end
end
registerAnonymousEventHandler("sysInstallPackage", handleInstall)

function handleShutdown()

    if bashmatic.configs.death.event ~= nil then killTrigger(bashmatic.configs.death.event) end
    bashmatic.configs.death.event = nil

    table.save(getMudletHomeDir() .. "/bashmatic.lua", bashmatic)

end
registerAnonymousEventHandler("sysExitEvent", handleShutdown)

function handleLoadEvent()

    if io.exists(getMudletHomeDir() .. "/bashmatic.lua") then
        table.load(getMudletHomeDir() .. "/bashmatic.lua", bashmatic)
    end

    if bashmatic.configs.death.phrase ~= nil then
        bashmatic.configs.death.event = tempTrigger(bashmatic.configs.death.phrase,handleMobDeath)
    end

    cecho("<DarkSlateBlue>BASHMATIC: Loaded settings. Let's bash some mobs.")

end
registerAnonymousEventHandler("sysLoadEvent", handleLoadEvent)

function bmDisable()
    bashmatic.enabled = false
end

function toggleBashing(status)
    bashmatic.bashing = status
end

function handleMobDeath()
    if bashmatic.configs.commands.after then
        send(bashmatic.configs.commands.after)
    end
    toggleBashing(false)
    if bashmatic.configs.enabled then
        send("look")
    end
end

function handleDuringTick()
    send(bashmatic.configs.commands.during)
    if bashmatic.configs.bashing then tempTimer(bashmatic.configs.commands.duringTime,handleDuringTick) end
end

-- Functions for handling inline matching

function handleBashingMatches(match)
    if not bashmatic.configs.enabled then
        -- bashing is disabled, so clear the triggers
        for _, tid in ipairs(bashmatic.hunting.events) do
            killTrigger(tid)
        end
        bashmatic.hunting.events = {}
    else
        if bashmatic.configs.bashing then
            -- we're currently bashing something, so reup the trigger and wait
            bmCreateTrigger(match)
        else
            if bashmatic.configs.commands.before then send(bashmatic.configs.commands.before) end
            send(bashmatic.configs.commands.start .. match)
            if bashmatic.configs.commands.during then tempTimer(bashmatic.configs.commands.duringTime,handleDuringTick) end
        end
    end
end

function bmCreateTrigger(keyword)
    table.insert(bashmatic.hunting.events, tempTrigger(keyword, handleBashingMatches(keyword), 1))
end

-- Functions for handling GMCP matching

-- Functions for handling adding targets

function inlineAddTarget(target, area)
    if area == nil then 
        cecho("<DarkSlateBlue>BASHMATIC: Added " .. matches[2] .. " to hunting targets.<reset>") 
        if not table.contains(bashmatic.hunting.targets, matches[2]) then
            table.insert(bashmatic.hunting.targets, matches[2])
        end
    end

    if area ~= nil then 
        cecho("<DarkSlateBlue>BASHMATIC: Added " .. matches[2] .. " to hunting targets in " .. area .. ".<reset>")
        if table.contains(bashmatic.hunting.targets, area) then
            table.insert(bashmatic.hunting.targets.area, matches[2])
        else
            bashmatic.hunting.targets.area = {}
            table.insert(bashmatic.hunting.targets.area, matches[2])
        end
    end
    
    if not bashmatic.configs.bashing then
        if bashmatic.configs.enabled then
            bmCreateTrigger(matches[2])
        end
    end
end